#define DDS_REF 180000000
#define BASE_FREQ 2000000

#define POT_CS    5
#define POT_SCK   6
#define POT_SI    7

#define DDS_CLOCK  8
#define DDS_LOAD   9
#define DDS_DATA   10
#define DDS_RESET  11

#define ANALOG_REFE A7
#define ANALOG_AMPL A6
#define ANALOG_PHAS A5

int flag = 1;

float kr = 1024.0;
float ka = 1024.0;
float kp = 1024.0;

float amp = 1024.0;
float phi = 1024.0;

int pot = 1;

void setup()
{
  Serial.begin(115200);
  
  pinMode (DDS_CLOCK, OUTPUT);
  pinMode (DDS_LOAD,  OUTPUT);
  pinMode (DDS_DATA,  OUTPUT);
  pinMode (DDS_RESET, OUTPUT);
  
  pinMode (POT_CS,   OUTPUT);
  pinMode (POT_SCK,  OUTPUT);
  pinMode (POT_SI,   OUTPUT);

  analogReadResolution(12);
  
  digitalWrite(DDS_RESET, HIGH);
  digitalWrite(DDS_RESET, LOW);
  
  digitalWrite(DDS_CLOCK, HIGH);
  digitalWrite(DDS_CLOCK, LOW);
  
  digitalWrite(DDS_LOAD, HIGH);
  digitalWrite(DDS_LOAD, LOW);
}

void loop()
{ 
  if(flag == 1)
  {
    sendFrequency(int(BASE_FREQ));
    for(int j=1; j<2; j+=1)
    {
      for(int i=1; i<21; i+=1)
      {
        if(int(15.08*i) >= 255)
        {
          pot = 255;
        } else {
          pot = int(15.08*i);
        }
        sendPotentiometer(pot);
        sendFrequency(int(BASE_FREQ*i));
        Serial.println(i);
        delay(2000);
        read_input();
        Serial.println(amp,0);
        Serial.println(phi,0);
      }
    }
    flag = 0;
  }
}

void read_input()
{
  kr = (float) analogRead(ANALOG_REFE);
  ka = (float) analogRead(ANALOG_AMPL);
  kp = (float) analogRead(ANALOG_PHAS);
  amp = (int) ((float) 4096.0*(ka/kr));
  phi = (int) ((float) 4096.0*(kp/kr));
}

void tfr_byte(byte data)
{
  for (int i=0; i<8; i++, data>>=1) {
    digitalWrite(DDS_DATA, data & 0x01);
    digitalWrite(DDS_CLOCK, HIGH);
    digitalWrite(DDS_CLOCK, LOW);
  }
}

void pot_byte(byte data)
{
  for (int i=0; i<8; i++, data>>=1) {
    digitalWrite(POT_SI, data & 0x01);
    digitalWrite(POT_SCK, HIGH);
    digitalWrite(POT_SCK, LOW);
  }
}

void sendFrequency(double frequency) {
  digitalWrite(DDS_LOAD, LOW);
  int32_t freq = frequency * 4294967295/125000000;  // 125 MHz clock on 9850
  for (int b=0; b<4; b++, freq>>=8) {
    tfr_byte(freq & 0xFF);
  }
  tfr_byte(0x000);
  digitalWrite(DDS_LOAD, HIGH);
}

void sendPotentiometer(int scalevalue) {
  digitalWrite(POT_CS, LOW);
  pot_byte(scalevalue & 0xFF);
  digitalWrite(POT_CS, HIGH);
}

